import { Component, ChangeDetectionStrategy, ViewChild, Input, Renderer2, Output, EventEmitter, ChangeDetectorRef } from '@angular/core';
import { readFileAsDataURL } from '../utils/dom-utils';
import { Http } from '@angular/http';
import { formatFileSize, formatString } from '../utils/lange-utils';
import { RebirthNGConfig } from '../rebirth-ng.config';
var FileUploadComponent = (function () {
    function FileUploadComponent(rebirthNGConfig, renderer, http, changeDetectorRef) {
        this.rebirthNGConfig = rebirthNGConfig;
        this.renderer = renderer;
        this.http = http;
        this.changeDetectorRef = changeDetectorRef;
        this.selectFilesChange = new EventEmitter();
        this.fileUploadSuccess = new EventEmitter();
        this.fileUploadError = new EventEmitter();
        this.removeFiles = new EventEmitter();
        this.uploadFilesChange = new EventEmitter();
        this.uploadFiles = [];
        this.selectFiles = [];
        this.errors = [];
        this.fileSizeErrorMessage = this.rebirthNGConfig.fileUpload.fileSizeErrorMessage;
        this.fileTypeErrorMessage = this.rebirthNGConfig.fileUpload.fileTypeErrorMessage;
        this.uploadParamName = this.rebirthNGConfig.fileUpload.uploadParamName;
        this.previewWidth = this.rebirthNGConfig.fileUpload.previewWidth;
        this.imgPreview = this.rebirthNGConfig.fileUpload.imgPreview;
        this.chooseButton = this.rebirthNGConfig.fileUpload.chooseButton;
        this.uploadButton = this.rebirthNGConfig.fileUpload.uploadButton;
        this.cancelButton = this.rebirthNGConfig.fileUpload.cancelButton;
    }
    FileUploadComponent.prototype.ngAfterViewInit = function () {
        if (this.accept) {
            this.renderer.setProperty(this.fileInput.nativeElement, 'accept', this.accept);
        }
        this.renderer.setProperty(this.fileInput.nativeElement, 'multiple', this.multiple);
    };
    FileUploadComponent.prototype.isMoreThanMaxItems = function () {
        if (!this.multiple) {
            return this.selectFiles.length >= 1;
        }
        return this.maxItems && (this.selectFiles.length >= this.maxItems);
    };
    FileUploadComponent.prototype.clearErrors = function () {
        this.errors = [];
    };
    FileUploadComponent.prototype.onDropFiles = function ($event) {
        $event.stopPropagation();
        $event.preventDefault();
        var files = $event.dataTransfer.files;
        if (files && files.length) {
            this.handleFileChoose(files);
        }
    };
    FileUploadComponent.prototype.addNewFile = function ($event) {
        $event.stopPropagation();
        this.fileInput.nativeElement.value = null;
        this.fileInput.nativeElement.click(); // simulate file input event
    };
    FileUploadComponent.prototype.newFileChoose = function (fileInput) {
        this.clearErrors();
        if (fileInput.files && fileInput.files.length) {
            this.handleFileChoose(fileInput.files);
        }
    };
    FileUploadComponent.prototype.removeAllSelectedFiles = function () {
        var files = this.selectFiles;
        this.selectFiles = [];
        this.clearErrors();
        this.removeFiles.emit(files);
    };
    FileUploadComponent.prototype.onRemoveFile = function (fileItem) {
        this.selectFiles = this.selectFiles.filter(function (item) { return item !== fileItem; });
        this.removeFiles.emit([fileItem]);
    };
    FileUploadComponent.prototype.uploadAllFiles = function () {
        var _this = this;
        this.clearErrors();
        this.selectFiles.map(function (fileItem) {
            return _this.uploadFile(fileItem);
        });
    };
    FileUploadComponent.prototype.uploadFile = function (fileItem) {
        var _this = this;
        var formData = new FormData();
        formData.append(this.uploadParamName, fileItem.file);
        return this.http.post(this.uploadUrl, formData, this.uploadRequestOptions)
            .map(function (res) { return res.json(); })
            .subscribe(function (res) {
            fileItem.uploadResponse = res;
            _this.selectFiles = _this.selectFiles.filter(function (item) { return item !== fileItem; });
            _this.uploadFiles = _this.uploadFiles.concat([fileItem]);
            _this.fileUploadSuccess.emit(fileItem);
            _this.uploadFilesChange.emit(_this.uploadFiles);
            _this.changeDetectorRef.markForCheck();
        }, function (error) {
            _this.errors.push(fileItem.name + ": Upload error: " + error);
            _this.fileUploadError.emit({
                name: fileItem.name,
                displaySize: fileItem.displaySize,
                dataUrl: fileItem.dataUrl,
                file: fileItem.file,
                uploadResponse: error
            });
            _this.changeDetectorRef.markForCheck();
        });
    };
    FileUploadComponent.prototype.handleFileChoose = function (uploadFiles) {
        var _this = this;
        var files = this.validFiles(Array.from(uploadFiles));
        this.mapFileModel(files)
            .then(function (fileModels) {
            _this.selectFiles = _this.selectFiles.concat(fileModels);
            _this.selectFilesChange.emit(_this.selectFiles);
            _this.changeDetectorRef.markForCheck();
            return fileModels || [];
        })
            .then(function (fileModels) {
            if (_this.autoUpload) {
                return fileModels.map(function (fileItem) { return _this.uploadFile(fileItem); });
            }
        });
    };
    FileUploadComponent.prototype.validFiles = function (files) {
        var _this = this;
        if (this.maxItems && (this.selectFiles.length + files.length > this.maxItems)) {
            files = files.slice(0, this.maxItems - this.selectFiles.length);
        }
        return files.filter(function (file) {
            return _this.validFile(file);
        });
    };
    FileUploadComponent.prototype.validFile = function (file) {
        var errors = [];
        if (this.maxFileSize && file.size > this.maxFileSize) {
            errors.push(formatString(this.fileSizeErrorMessage, file.name, formatFileSize(this.maxFileSize)));
        }
        if (this.accept && !this.validFileType(file)) {
            errors.push(formatString(this.fileTypeErrorMessage, file.name, this.accept));
        }
        (_a = this.errors).push.apply(_a, errors);
        return !errors.length;
        var _a;
    };
    FileUploadComponent.prototype.validFileType = function (file) {
        return this.accept.split(',').some(function (type) {
            return new RegExp("^" + type.replace(/\*/g, '.*') + "$").test(file.type);
        });
    };
    FileUploadComponent.prototype.mapFileModel = function (files) {
        return Promise.all(files.map(function (file) {
            return readFileAsDataURL(file)
                .then(function (dataUrl) { return ({ dataUrl: dataUrl, name: file.name, file: file, displaySize: formatFileSize(file.size) }); });
        }));
    };
    return FileUploadComponent;
}());
export { FileUploadComponent };
FileUploadComponent.decorators = [
    { type: Component, args: [{
                selector: 're-file-upload',
                template: "\n    <input #file type=\"file\" (change)=\"newFileChoose(file)\" style=\"display: none\"><ng-template #defaultToolbarTemplate let-toolbar=\"\"><div class=\"file-upload-toolbar\"><button class=\"btn btn-primary\" (click)=\"toolbar.addNewFile($event)\" [disabled]=\"toolbar.isMoreThanMaxItems()\"><i class=\"glyphicon glyphicon-plus\"></i> {{toolbar.chooseButton}}</button> <button class=\"btn btn-info\" [disabled]=\"!toolbar.selectFiles?.length\" (click)=\"toolbar.uploadAllFiles()\"><i class=\"glyphicon glyphicon-upload\"></i> {{toolbar.uploadButton}}</button> <button class=\"btn btn-danger\" [disabled]=\"!toolbar.selectFiles?.length\" (click)=\"toolbar.removeAllSelectedFiles()\"><i class=\"glyphicon glyphicon-trash\"></i> {{toolbar.cancelButton}}</button></div></ng-template><ng-template #defaultPreviewTemplate let-preview=\"\"><re-file-upload-preview [selectFiles]=\"preview.selectFiles\" [imgPreview]=\"preview.imgPreview\" [previewWidth]=\"preview.previewWidth\" (removeFile)=\"preview.onRemoveFile($event)\"></re-file-upload-preview><re-file-upload-preview [selectFiles]=\"preview.uploadFiles\" [imgPreview]=\"preview.imgPreview\" [previewWidth]=\"preview.previewWidth\" [uploaded]=\"true\"></re-file-upload-preview></ng-template><div class=\"file-upload {{cssClass}}\" reDroppable (onDrop)=\"onDropFiles($event)\"><ng-template [ngTemplateOutlet]=\"toolbarTemplate || defaultToolbarTemplate\" [ngOutletContext]=\"{ $implicit: this}\"></ng-template><re-alert-box class=\"file-upload-error-box\" type=\"danger\" [closable]=\"true\" (close)=\"clearErrors()\" *ngIf=\"errors.length\"><ul><li *ngFor=\"let error of errors\">{{error}}</li></ul></re-alert-box><ng-template [ngTemplateOutlet]=\"previewTemplate || defaultPreviewTemplate\" [ngOutletContext]=\"{ $implicit: this}\"></ng-template></div>\n  ",
                styles: ["\n    .file-upload-toolbar{display:block;margin:1rem 0}\n  "],
                changeDetection: ChangeDetectionStrategy.OnPush,
                exportAs: 'fileUpload'
            },] },
];
/** @nocollapse */
FileUploadComponent.ctorParameters = function () { return [
    { type: RebirthNGConfig, },
    { type: Renderer2, },
    { type: Http, },
    { type: ChangeDetectorRef, },
]; };
FileUploadComponent.propDecorators = {
    'accept': [{ type: Input },],
    'multiple': [{ type: Input },],
    'autoUpload': [{ type: Input },],
    'maxItems': [{ type: Input },],
    'maxFileSize': [{ type: Input },],
    'uploadUrl': [{ type: Input },],
    'uploadParamName': [{ type: Input },],
    'uploadRequestOptions': [{ type: Input },],
    'imgPreview': [{ type: Input },],
    'previewWidth': [{ type: Input },],
    'cssClass': [{ type: Input },],
    'fileSizeErrorMessage': [{ type: Input },],
    'fileTypeErrorMessage': [{ type: Input },],
    'chooseButton': [{ type: Input },],
    'uploadButton': [{ type: Input },],
    'cancelButton': [{ type: Input },],
    'toolbarTemplate': [{ type: Input },],
    'previewTemplate': [{ type: Input },],
    'selectFilesChange': [{ type: Output },],
    'fileUploadSuccess': [{ type: Output },],
    'fileUploadError': [{ type: Output },],
    'removeFiles': [{ type: Output },],
    'uploadFilesChange': [{ type: Output },],
    'uploadFiles': [{ type: Input },],
    'fileInput': [{ type: ViewChild, args: ['file',] },],
};
//# sourceMappingURL=file-upload.component.js.map